; Copyright (c) 2004-2010, Laurent Jorda
;   All rights reserves

FUNCTION compute_nac_boresight, los_st, bor, DEBUG=debug, IFAIL=ifail

;
; initialization
;
d     = keyword_set(debug)
ct    = 180.0d0/!dpi
ifail = 0

if (d) then print,'M: COMPUTE_BORESIGHT_01 - BEGIN.'

;
; extract parameters from 'bor' and 'los_st' structures
;
tpix      = double(bor.pixsize)
doff      = dblarr(3)
doff(0:1) = bor.offset
doff(2)   = bor.deltaroll
dcos      = dblarr(3)
dcos      = double([los_st.ra,los_st.dec,los_st.roll])
dcor      = dblarr(3)

;
; computing camera roll angle
;
dcor(2)   = dcos(2)-doff(2)

if (d) then print,'M:Camera ROLL = ',dcor(2)*ct,' deg.'

costhe = cos(dcor(2))
sinthe = sin(dcor(2))

;
; computing angular offsets
;
dang    = dblarr(2)
dang(0) = (-doff(0)*costhe+doff(1)*sinthe)*tpix(0)
dang(1) = (-doff(0)*sinthe+doff(1)*costhe)*tpix(1)

if (d) then begin

    print,'M:Angular offset along X = ',dang(0)*ct*3600.0d0,' arcsec.'
    print,'M:Angular offset along Y = ',dang(1)*ct*3600.0d0,' arcsec.'

endif

;
; computing line-of-sight of reference pixel
;
dcor(1) = dcos(1)-dang(1)
dcor(0) = dcos(0)-dang(0)/cos(dcor(1))

if (dcor(0) lt 0.0d0) then dcor(0) = dcor(0)+1.0d0*pi

if (d) then begin

    print,'M:Reference pixel RA  = ',dcor(0)*ct,' deg.'
    print,'M:Reference pixel DEC = ',dcor(1)*ct,' deg.'

endif

;
; create structure 'los_rp'
;
los_rp = { $
           ra       : dcor(0), $
           dec      : dcor(1), $
           roll     : dcor(2), $
           ra_deg   : dcor(0)*ct, $
           dec_deg  : dcor(1)*ct, $
           roll_deg : dcor(2)*ct }

;
; end of routine
;
if (d) then print,'M: COMPUTE_BORESIGHT_01 - END.'

RETURN, los_rp
END
