; REPAIR.PRO  by  R. KRAMM  12-FEB-2003
; =====================================
;
; RMV_CHN_NOI.PRO  by  R. Kramm  23-JUL-2003, 04-NOV-2004
; =======================================================
;
;+
; NAME:
;   FUNCTION osiris_repair, header, image
;
; PURPOSE:
;   Remove coherent noise from image
;
; CATEGORY:
;   Osiris CCD Calibration
;
; CALLING SEQUENCE:
;  Result = osiris_rmv_chn_noi(header, image)
;
; INPUTS:
;   header:   The PDS image header of the image to be corrected
;   image:      The image data of the image to be corrected
;
; KEYWORD PARAMETERS:
;   NONE
;
; OUTPUTS:
;   amp:		The amplitude of the CHN in x and y
;   anz:		?
;
; RETURNS:
;   image with same dimensions as the input image containing the noise component
;   On error -1
;
; RESTRICTIONS:
;	None
;
; EXAMPLE:
;   p_read, pickfile(), h, d             ; read in raw PDS image
;   noise = osiris_rmv_chn_noi(h, d, amp, anz)
;   d = d - noise
;
; MODIFICATION HISTORY:
;   Written by:  			R. Kramm   12-FEB-2003
;   Made PDS compatible		S.F. Hviid 2-JUL-2005
;-

PRO OSIRIS_REPAIR, H,P, HH,PP, pixel_=pixel_, filter_=filter_, quiet_=quiet_,		$
						help_=help_
;
@C_HEADER.INC
;
;					Removal of blocked or partially blocked  columns.
;					Removal of hot pixels if Keyword "/pixel" is set.
;					Removal of cosmics   if Keyword "/filter" is set.
; ---------------------------------------------------------------------------
;
IF (N_PARAMS(0) LT 4) OR KEYWORD_SET(help_)  THEN  BEGIN
	PRINT, ' '
	PRINT, ' Call is :  REPAIR, H,P, HH,PP [, /pixel, /help]  '
	PRINT, ' -------------------------------------------------'
	PRINT, ' Removal of blocked or partially blocked  columns.'
	PRINT, ' Removal of hot pixels if Keyword "/pixel" is set.'
	PRINT, ' Removal of cosmics   if Keyword "/filter" is set.
	PRINT, ' -------------------------------------------------'
	PRINT, ' Exit.'
	PRINT, ' '
	RETURN
ENDIF
;
; ---------------------------------------------------------------------------
;
PROC = 'REPAIR'		&  VERS = 'V1.0'
;
HH = H				&  PP = P
;
; ---------------------------------------------------------------------------
;
SEARCH_MISSING, HH,PP, MIS
;
IF (MIS NE 0)  THEN  BEGIN
	PRINT, ' '
	PRINT, '     MIS = '+STRTRIM(MIS,2)
	STOP,  ' *** REPAIR. STOP. Missing data. ***'
	RETURN
ENDIF
;
; ---------------------------------------------------------------------------
;
BAD_PIX = 0		&  IF KEYWORD_SET(pixel_)   THEN  BAD_PIX = 1
				   IF KEYWORD_SET(filter_)  THEN  BAD_PIX = 1
;
; ---------------------------------------------------------------------------
;
OFS = 220.01
;
GAIN = 1. / ((4.*D_GAIN(H)) > 1.)			; normal: 1,  with GAIN_SET: 1/4.
;
CCD = D_VALUE(H, H_SENSOR_ID)
TMP = D_VALUE(H, H_CCDCOLDT)
CHL = D_VALUE(H, H_CHANNEL)
;
DIT = D_VALUE(H, H_DITHER)
BIN = D_VALUE(H, H_BINNING)
;
XL  = D_VALUE(H, H_COORD_XL)	&  YL  = D_VALUE(H, H_COORD_YL)
XU  = D_VALUE(H, H_COORD_XU)	&  YU  = D_VALUE(H, H_COORD_YU)
DX  = D_VALUE(H, H_XSIZE)		&  DY  = D_VALUE(H, H_YSIZE)
;
; ---------------------------------------------------------------------------
;														 left and right edges
IF (XL LE 0L) AND (XU GT 1L)  THEN  BEGIN
	PX = 0L		&  X = (PX - XL)/BIN
	SCL = (TOTAL(PP(X,*)) - OFS) / (TOTAL(PP(X+2,*)) - OFS)
	IF (SCL GT 1.1)  THEN  PP(X,0) = (PP(X,*)-OFS)/SCL+OFS
ENDIF
;
IF (XU GE 2048L) AND (XL LT 2047L)  THEN  BEGIN
	PX = 2047L	&  X = (PX-XL)/BIN
	SCL = D_MEAN((PP(X,*) - OFS) / (PP(X-1,*) - OFS))
	IF (SCL GT 1.1)  THEN  PP(X,0) = (PP(X,*)-OFS)/SCL+OFS
ENDIF
;
; ---------------------------------------------------------------------------
;
IF (CCD EQ 241)  THEN  BEGIN
	IF (BAD_PIX EQ 0)  THEN  GOTO, NXT
	;
	PX = 970L	&  PY = 1630L	&  X = (PX-XL)/BIN	&  Y = (PY-YL)/BIN
	LIM = (26700. * FLOAT(BIN)^2 / GAIN + 200.) * FLOAT(BIN)
	;
	IF (X<Y<(DX-X)<(DY-Y) GT 0) THEN IF (MAX(P(X,Y:*)) GT LIM)  THEN  BEGIN
		PP(X,Y) = (PP(X-1,Y:*) + PP(X+1,Y:*))/2
		;
		POS = WHERE((ABS(P-PP) GT 0) AND (ABS(P-PP) LT 40))
		IF (TOTAL(POS) GE 0.) THEN  PP(POS) = P(POS)
	ENDIF
	;
	GOTO, NXT
ENDIF
;
; ---------------------------------------------------------------------------
;
IF (CCD EQ 242)  THEN  BEGIN
	IF (BAD_PIX EQ 0)  THEN  GOTO, NXT
	;
	PX = 119L	&  PY = 1663L	&  X = (PX-XL)/BIN	&  Y = (PY-YL)/BIN
	IF (X<Y<(DX-X)<(DY-Y) GT 0)  THEN  PP(X,Y) = (PP(X-1,Y:*)+PP(X+1,Y:*))/2
	;
	POS = WHERE((ABS(P-PP) GT 0) AND (ABS(P-PP) LT 50))
	IF (TOTAL(POS) GE 0.) THEN  PP(POS) = P(POS)
	;
	GOTO, NXT
ENDIF
;
; ---------------------------------------------------------------------------
;
IF (CCD EQ 243)  THEN  BEGIN
	;
	; -----------------------------------------------------------------------
	;															bright column
	PX = 995L		&  XCL = (PX-XL)/BIN
	;
	X = [ XCL-1L, XCL, (XCL+3L)<(DX-1L) ]						  ; Cha. A/AB
	IF (STRPOS(CHL,'A') LT 0L)  THEN  X = REVERSE(X(1)-X)+X(1)	  ; Channel B
	;
	IF (X(1)<(DX-X(1)) GT 0)  THEN  BEGIN
		IF (YL EQ 0)  THEN  PP(X(1),1) = PP(X(1),2)				  ; 2. px bad
		;
		PP(X(1),0)=(PP(X(0),1)+PP(X(2),1))/2  ; bottom px
		;
		COL = ROTATE(PP(X(1),1:*)-(PP(X(0),1:*)+PP(X(2),1:*))/2,3)
		CAD = INDGEN(DY-1)
		AAA = POLY_FIT(CAD, COL, 2, FLAT)
		AAA = POLY_FIT(CAD, COL > (FLAT-50) < (FLAT+50), 2, FLAT)
		PP(X(1),1) = PP(X(1),1:*)-ROTATE(FLAT,1)

		PP(X(1),0) = PP(X(1),*) - D_MEAN(PP(X(1),*)-(PP(X(0),*)+PP(X(2),*))/2)
	ENDIF
	;
	;					   neighbouring columns: Ch. A,AB: right / Ch. B: left
	;
	X(1) = X(1)+1+2*STRPOS(CHL,'A')		&  IF (X(1)<(DX-X(1)) GT 0)  THEN  	$
		PP(X(1),0) = PP(X(1),*) - D_MEAN(PP(X(1),*)-(PP(X(0),*)+PP(X(2),*))/2)
	;
	X(1) = X(1)+1+2*STRPOS(CHL,'A')		&  IF (X(1)<(DX-X(1)) GT 0)  THEN	$
		PP(X(1),0) = PP(X(1),*) - D_MEAN(PP(X(1),*)-(PP(X(0),*)+PP(X(2),*))/2)
	;
	IF (BIN GE 2)  THEN  IF (XCL<(DX-XCL) GT 0)  THEN	$
		PP(XCL,0) = (PP(XCL-1,*)+PP(XCL+1,*))/2
	;
	; -----------------------------------------------------------------------
	; Removal of hot pixels especially found at higher temperatures
	; Removal here only if  amplitude > 50 DU  above l&r neighbours
	; -----------------------------------------------------------------------
	;
	IF (BAD_PIX EQ 0)  THEN  GOTO, NXT
	;
	PX = PP
	;
	PPS = [[ 885,1544, 935, 554, 717, 804, 436,  55,1866,1097], $
	       [ 779,1304,1466,1449,1057, 255, 730, 501,1785, 292]]
	;
	FOR J = 0, N_ELEMENTS(PPS(*,0))-1 DO BEGIN
		X = (PPS(J,0)-XL)/BIN		&  Y = (PPS(J,1)-YL)/BIN
		IF (X<Y<(DX-X)<(DY-Y) GT 0)  THEN  PX(X,Y) = (PX(X-1,Y)+PX(X+1,Y))/2
	ENDFOR
	;
	POS = WHERE(ABS(PP-PX) GT 50, CNT)
	IF (CNT GT 0)  THEN  PP(POS)=PX(POS)
	IF (CNT EQ 0)  THEN  $
	IF NOT KEYWORD_SET(quiet_)  THEN  PRINT, ' No declared hot pixels found'
	;
	GOTO, NXT
ENDIF
;
; ---------------------------------------------------------------------------
;
STOP, ' *** REPAIR. NOT FM camera model. STOP ***'		&  RETURN
;
; ---------------------------------------------------------------------------
;
NXT:	D_APPROC, H,HH, PROC, VERS
;
; ---------------------------------------------------------------------------
;																  ; hot pixel
IF KEYWORD_SET(filter_)  THEN  BEGIN
	;
	SCL = 15. * ALOG10(TMP/120.) > 3.
	IF (DIT EQ 'Y')  THEN  FCT = 100.  ELSE  FCT = 200.
	IF (D_SHUTTER(H) EQ 0)  THEN  FCT = FCT/2.
	;
	KER = 2L*(((DX < DY)-1L)/2L)+1L
	;
	IF (KER GE 3L)  THEN  BEGIN
		MED = SMOOTH(MEDIAN(MEDIAN(PP,KER<11),KER<3), 3)
		POS = WHERE((ABS(PP-MED) - SCL*SQRT(MED-200.>0.)) GT SCL*FCT)
		IF (TOTAL(POS) NE -1.)  THEN  PP(POS) = MED(POS)
		;
		IF NOT KEYWORD_SET(quiet_)  THEN  $
		PRINT, ' --- '+STRTRIM(N_ELEMENTS(POS),2)+' hot pixel removed. ---'
	ENDIF
ENDIF
;
; ---------------------------------------------------------------------------
;
RETURN
END