; Copyright (c) 2006, Stubbe F. Hviid
;   All rights reserved
;+
; NAME:
;   PRO osiris_cal_level_2b
;
; PURPOSE:
;   Generate cosmic ray event cleaned image based on a level2 OSIRIS archive node
;
; CATEGORY:
;   Osiris CCD Calibration
;
; CALLING SEQUENCE:
;  PRO osiris_cal_level_2b
;
; INPUTS:
;   level1_dir		- Directory contains level1 data
; 	level2_dir		- Directory containing the corresponding level2 data
;	out_dir			- the directory where the images are saved
;
; OUTPUTS:
;   None
;
; RESTRICTIONS:
;   function requires that the paths defined in get_osiris_environment.pro has been set to correctly
;   point to the root directory of the external calibration files
;
; EXAMPLE:
;   p_read, pickfile(), h, d             ; read in raw PDS image
;   osiris_badpixel_removal, h, d
;
;-

PRO osiris_cal_level_2b, level1_dir, level2_dir, out_dir

	files = findfile(level2_dir + '\*.img')

	window, 1, xsize = 1024, ysize = 512
	for i=0, n_elements(files)-1 do begin
		fd = p_parse_file_path(files[i])
		p = strpos(fd.name, '_ID') + 3
		level1_name = strmid(fd.name, 0, p) + '1' + strmid(fd.name, p+1)
		level2b_name = strmid(fd.name, 0, p) + 'X' + strmid(fd.name, p+1)

		p_read, files[i], hcal, ical
		sd = size(ical, /dim)

		if sd[0] ge 64 AND sd[1] ge 64 then begin
			p_read, level1_dir + '\' + level1_name + '.img', h, iraw

			mask = find_cosmics(iraw)
			fimage = remove_cosmics(ical, mask)

			!p.multi = [0,2,1]

			fw_contour, alog10(ical*1.0e9 > 0.0001)
			fw_contour, alog10(fimage*1.0e9 > 0.0001)

			p_setvalue, hcal, 'PROCESSING_LEVEL_ID', '2b'
			p_setvalue, hcal, 'PROCESSING_LEVEL_DESC', 'Calibrated data with cosmics removed'
			p_setvalue, hcal, 'FILE_NAME', level2b_name + '.img'

			p_write, out_dir + '\' + level2b_name + '.img', hcal, fimage
			write_jpeg, out_dir + '\' + level2b_name + '_mask.jpg', bytscl(mask)
		endif

		print, level1_name
	endfor


END